// NOTE: This document was originally generated by flatc, however, modifications have been made to this file
// to resolve build errors.

library promissory;

import 'dart:typed_data' show Uint8List;
import 'package:flat_buffers/flat_buffers.dart' as fb;

///  Cryptographic backend for signatures
class SignatureSystem {
  final int value;
  const SignatureSystem._(this.value);

  factory SignatureSystem.fromValue(int value) {
    if (!values.containsKey(value)) {
      throw StateError(
          'Invalid value $value for bit flag enum SignatureSystem');
    }
    return values[value]!;
  }

  static const int minValue = 0;
  static const int maxValue = 0;
  static bool containsValue(int value) => values.containsKey(value);

  static const SignatureSystem Ed25519 = const SignatureSystem._(0);
  static const Map<int, SignatureSystem> values = {
    0: Ed25519,
  };

  static const fb.Reader<SignatureSystem> reader =
      const _SignatureSystemReader();

  @override
  String toString() {
    return 'SignatureSystem{value: $value}';
  }
}

class _SignatureSystemReader extends fb.Reader<SignatureSystem> {
  const _SignatureSystemReader();

  @override
  int get size => 1;

  @override
  SignatureSystem read(fb.BufferContext bc, int offset) =>
      new SignatureSystem.fromValue(const fb.Int8Reader().read(bc, offset));
}

class Version {
  Version._(this._bc, this._bcOffset);

  static const fb.Reader<Version> reader = const _VersionReader();

  final fb.BufferContext _bc;
  final int _bcOffset;

  int get major => const fb.Uint8Reader().read(_bc, _bcOffset + 0);
  int get minor => const fb.Uint8Reader().read(_bc, _bcOffset + 1);
  int get patch => const fb.Uint8Reader().read(_bc, _bcOffset + 2);

  @override
  String toString() {
    return 'Version{major: $major, minor: $minor, patch: $patch}';
  }
}

class _VersionReader extends fb.StructReader<Version> {
  const _VersionReader();

  @override
  int get size => 3;

  @override
  Version createObject(fb.BufferContext bc, int offset) =>
      new Version._(bc, offset);
}

class VersionBuilder {
  VersionBuilder(this.fbBuilder) {
    assert(fbBuilder != null);
  }

  final fb.Builder fbBuilder;

  int finish(int major, int minor, int patch) {
    fbBuilder.putUint8(patch);
    fbBuilder.putUint8(minor);
    fbBuilder.putUint8(major);
    return fbBuilder.offset;
  }
}

class VersionObjectBuilder extends fb.ObjectBuilder {
  final int _major;
  final int _minor;
  final int _patch;

  VersionObjectBuilder({
    required int major,
    required int minor,
    required int patch,
  })  : _major = major,
        _minor = minor,
        _patch = patch;

  /// Finish building, and store into the [fbBuilder].
  @override
  int finish(fb.Builder fbBuilder) {
    assert(fbBuilder != null);

    fbBuilder.putUint8(_patch);
    fbBuilder.putUint8(_minor);
    fbBuilder.putUint8(_major);
    return fbBuilder.offset;
  }

  /// Convenience method to serialize to byte list.
  @override
  Uint8List toBytes([String fileIdentifier = ""]) {
    fb.Builder fbBuilder = new fb.Builder();
    int offset = finish(fbBuilder);
    fbBuilder.finish(offset, fileIdentifier);
    return fbBuilder.buffer;
  }
}

///  Signature block hash list. Each block hash is determined based on the previous block hash.
class SignatureBlock {
  SignatureBlock._(this._bc, this._bcOffset);
  factory SignatureBlock(List<int> bytes) {
    fb.BufferContext rootRef = new fb.BufferContext.fromBytes(bytes);
    return reader.read(rootRef, 0);
  }

  static const fb.Reader<SignatureBlock> reader = const _SignatureBlockReader();

  final fb.BufferContext _bc;
  final int _bcOffset;

  ///  Public key of the signature from the wallet holding the promissory
  List<int> get publicKey => const fb.ListReader<int>(const fb.Uint8Reader())
      .vTableGet(_bc, _bcOffset, 4, List.empty());

  ///  Sha2 sha256 Hash of the current signature block
  ///  sha256(previous hash + current [recipient] public key)
  ///  Genesis hash is sha256(mint public key + the UUID of the promissory + issuance timestamp + amount + decimals + currency code)
  List<int> get hash => const fb.ListReader<int>(const fb.Uint8Reader())
      .vTableGet(_bc, _bcOffset, 6, List.empty());

  ///  Previous Signature Block
  SignatureBlock get previous =>
      // Return itself if there are no previous blocks;
      SignatureBlock.reader.vTableGet(_bc, _bcOffset, 8, this);

  ///  Timestamp when the new block was created
  int get timestamp => const fb.Int64Reader().vTableGet(_bc, _bcOffset, 10, 0);

  @override
  String toString() {
    return 'SignatureBlock{publicKey: $publicKey, hash: $hash, previous: $previous, timestamp: $timestamp}';
  }
}

class _SignatureBlockReader extends fb.TableReader<SignatureBlock> {
  const _SignatureBlockReader();

  @override
  SignatureBlock createObject(fb.BufferContext bc, int offset) =>
      new SignatureBlock._(bc, offset);
}

class SignatureBlockBuilder {
  SignatureBlockBuilder(this.fbBuilder) {
    assert(fbBuilder != null);
  }

  final fb.Builder fbBuilder;

  void begin() {
    fbBuilder.startTable(4);
  }

  int addPublicKeyOffset(int offset) {
    fbBuilder.addOffset(0, offset);
    return fbBuilder.offset;
  }

  int addHashOffset(int offset) {
    fbBuilder.addOffset(1, offset);
    return fbBuilder.offset;
  }

  int addPreviousOffset(int offset) {
    fbBuilder.addOffset(2, offset);
    return fbBuilder.offset;
  }

  int addTimestamp(int timestamp) {
    fbBuilder.addInt64(3, timestamp);
    return fbBuilder.offset;
  }

  int finish() {
    return fbBuilder.endTable();
  }
}

class SignatureBlockObjectBuilder extends fb.ObjectBuilder {
  final List<int> _publicKey;
  final List<int> _hash;
  final SignatureBlockObjectBuilder? _previous;
  final int _timestamp;

  SignatureBlockObjectBuilder({
    required List<int> publicKey,
    required List<int> hash,
    SignatureBlockObjectBuilder? previous,
    required int timestamp,
  })  : _publicKey = publicKey,
        _hash = hash,
        _previous = previous,
        _timestamp = timestamp;

  /// Finish building, and store into the [fbBuilder].
  @override
  int finish(fb.Builder fbBuilder) {
    assert(fbBuilder != null);
    final int publicKeyOffset = fbBuilder.writeListUint8(_publicKey);
    final int hashOffset = fbBuilder.writeListUint8(_hash);
    final int previousOffset = _previous?.getOrCreateOffset(fbBuilder) ?? 0;

    fbBuilder.startTable(4);
    if (publicKeyOffset != null) {
      fbBuilder.addOffset(0, publicKeyOffset);
    }
    if (hashOffset != null) {
      fbBuilder.addOffset(1, hashOffset);
    }
    if (previousOffset != null) {
      fbBuilder.addOffset(2, previousOffset);
    }
    fbBuilder.addInt64(3, _timestamp);
    return fbBuilder.endTable();
  }

  /// Convenience method to serialize to byte list.
  @override
  Uint8List toBytes([String fileIdentifier = ""]) {
    fb.Builder fbBuilder = new fb.Builder();
    int offset = finish(fbBuilder);
    fbBuilder.finish(offset, fileIdentifier);
    return fbBuilder.buffer;
  }
}

///  Root table for a promissory file.
class Promissory {
  Promissory._(this._bc, this._bcOffset);
  factory Promissory(List<int> bytes) {
    fb.BufferContext rootRef = new fb.BufferContext.fromBytes(bytes);
    return reader.read(rootRef, 0);
  }

  static const fb.Reader<Promissory> reader = const _PromissoryReader();

  final fb.BufferContext _bc;
  final int _bcOffset;

  ///  Semantic version of the promissory file, e.g. 0.1.0
  Version get version {
    VersionObjectBuilder builder =
        VersionObjectBuilder(major: 0, minor: 1, patch: 0);
    fb.BufferContext bc = fb.BufferContext.fromBytes(builder.toBytes());
    Version version = Version._(bc, 0);
    return Version.reader.vTableGet(_bc, _bcOffset, 4, version);
  }

  ///  Cryptographic algorithm used to sign and verify promissory file
  SignatureSystem get signatureSystem => new SignatureSystem.fromValue(
      const fb.Int8Reader().vTableGet(_bc, _bcOffset, 6, 0));

  ///  Universally Unique Identifier (UUID) (v4) of the promissory note
  List<int> get id => const fb.ListReader<int>(const fb.Uint8Reader())
      .vTableGet(_bc, _bcOffset, 8, List.empty());

  ///  A three-letter currency code, e.g. USD
  String get currencyCode =>
      const fb.StringReader().vTableGet(_bc, _bcOffset, 10, "");

  ///  Amount of the promissory, represented as a decimal expanded unsigned integer
  int get amount => const fb.Int64Reader().vTableGet(_bc, _bcOffset, 12, 0);

  ///  Decimal places of the amount, e.g. amount defined of 100 with 2 decimal places is equivalent to $1.00
  int get decimals => const fb.Uint32Reader().vTableGet(_bc, _bcOffset, 14, 0);

  ///  Issuing Authority public key identifier
  List<int> get authority => const fb.ListReader<int>(const fb.Uint8Reader())
      .vTableGet(_bc, _bcOffset, 16, List.empty());

  ///  Issuer public key
  List<int> get issuer => const fb.ListReader<int>(const fb.Uint8Reader())
      .vTableGet(_bc, _bcOffset, 18, List.empty());

  ///  Issuance date represented as a timestamp
  int get issuanceDate =>
      const fb.Int64Reader().vTableGet(_bc, _bcOffset, 20, 0);

  ///  Secured signature of the most recent block hash signed by the authority or delagated mint
  List<int> get authorizedSignature =>
      const fb.ListReader<int>(const fb.Uint8Reader())
          .vTableGet(_bc, _bcOffset, 22, List.empty());

  ///  Secured signature of the hash, signed by the previous [sender] signature block public key
  List<int> get securedSignature =>
      const fb.ListReader<int>(const fb.Uint8Reader())
          .vTableGet(_bc, _bcOffset, 24, List.empty());

  ///  Signature block of transactions
  SignatureBlock get signatureBlock => SignatureBlock.reader
      .vTableGet(_bc, _bcOffset, 26, SignatureBlock(List.empty()));

  ///  depth of the signature block, i.e. the entire number of times the promissory has been transferred
  int get blockDepth => const fb.Uint8Reader().vTableGet(_bc, _bcOffset, 28, 0);

  @override
  String toString() {
    return 'Promissory{version: $version, signatureSystem: $signatureSystem, id: $id, currencyCode: $currencyCode, amount: $amount, decimals: $decimals, authority: $authority, issuer: $issuer, issuanceDate: $issuanceDate, authorizedSignature: $authorizedSignature, securedSignature: $securedSignature, signatureBlock: $signatureBlock, blockDepth: $blockDepth}';
  }
}

class _PromissoryReader extends fb.TableReader<Promissory> {
  const _PromissoryReader();

  @override
  Promissory createObject(fb.BufferContext bc, int offset) =>
      new Promissory._(bc, offset);
}

class PromissoryBuilder {
  PromissoryBuilder(this.fbBuilder) {
    assert(fbBuilder != null);
  }

  final fb.Builder fbBuilder;

  void begin() {
    fbBuilder.startTable(13);
  }

  int addVersion(int offset) {
    fbBuilder.addStruct(0, offset);
    return fbBuilder.offset;
  }

  int addSignatureSystem(SignatureSystem signatureSystem) {
    fbBuilder.addInt8(1, signatureSystem.value);
    return fbBuilder.offset;
  }

  int addIdOffset(int offset) {
    fbBuilder.addOffset(2, offset);
    return fbBuilder.offset;
  }

  int addCurrencyCodeOffset(int offset) {
    fbBuilder.addOffset(3, offset);
    return fbBuilder.offset;
  }

  int addAmount(int amount) {
    fbBuilder.addInt64(4, amount);
    return fbBuilder.offset;
  }

  int addDecimals(int decimals) {
    fbBuilder.addUint32(5, decimals);
    return fbBuilder.offset;
  }

  int addAuthorityOffset(int offset) {
    fbBuilder.addOffset(6, offset);
    return fbBuilder.offset;
  }

  int addIssuerOffset(int offset) {
    fbBuilder.addOffset(7, offset);
    return fbBuilder.offset;
  }

  int addIssuanceDate(int issuanceDate) {
    fbBuilder.addInt64(8, issuanceDate);
    return fbBuilder.offset;
  }

  int addAuthorizedSignatureOffset(int offset) {
    fbBuilder.addOffset(9, offset);
    return fbBuilder.offset;
  }

  int addSecuredSignatureOffset(int offset) {
    fbBuilder.addOffset(10, offset);
    return fbBuilder.offset;
  }

  int addSignatureBlockOffset(int offset) {
    fbBuilder.addOffset(11, offset);
    return fbBuilder.offset;
  }

  int addBlockDepth(int blockDepth) {
    fbBuilder.addUint8(12, blockDepth);
    return fbBuilder.offset;
  }

  int finish() {
    return fbBuilder.endTable();
  }
}

class PromissoryObjectBuilder extends fb.ObjectBuilder {
  final VersionObjectBuilder _version;
  final SignatureSystem _signatureSystem;
  final List<int> _id;
  final String _currencyCode;
  final int _amount;
  final int _decimals;
  final List<int> _authority;
  final List<int> _issuer;
  final int _issuanceDate;
  final List<int> _authorizedSignature;
  final List<int> _securedSignature;
  final SignatureBlockObjectBuilder _signatureBlock;
  final int _blockDepth;

  PromissoryObjectBuilder({
    required VersionObjectBuilder version,
    required SignatureSystem signatureSystem,
    required List<int> id,
    required String currencyCode,
    required int amount,
    required int decimals,
    required List<int> authority,
    required List<int> issuer,
    required int issuanceDate,
    required List<int> authorizedSignature,
    required List<int> securedSignature,
    required SignatureBlockObjectBuilder signatureBlock,
    required int blockDepth,
  })  : _version = version,
        _signatureSystem = signatureSystem,
        _id = id,
        _currencyCode = currencyCode,
        _amount = amount,
        _decimals = decimals,
        _authority = authority,
        _issuer = issuer,
        _issuanceDate = issuanceDate,
        _authorizedSignature = authorizedSignature,
        _securedSignature = securedSignature,
        _signatureBlock = signatureBlock,
        _blockDepth = blockDepth;

  /// Finish building, and store into the [fbBuilder].
  @override
  int finish(fb.Builder fbBuilder) {
    assert(fbBuilder != null);
    final int idOffset = fbBuilder.writeListUint8(_id);
    final int currencyCodeOffset = fbBuilder.writeString(_currencyCode);
    final int authorityOffset = fbBuilder.writeListUint8(_authority);
    final int issuerOffset = fbBuilder.writeListUint8(_issuer);
    final int authorizedSignatureOffset =
        fbBuilder.writeListUint8(_authorizedSignature);
    final int securedSignatureOffset =
        fbBuilder.writeListUint8(_securedSignature);
    final int signatureBlockOffset =
        _signatureBlock.getOrCreateOffset(fbBuilder);

    fbBuilder.startTable(13);
    if (_version != null) {
      fbBuilder.addStruct(0, _version.finish(fbBuilder));
    }
    fbBuilder.addInt8(1, _signatureSystem.value);
    if (idOffset != null) {
      fbBuilder.addOffset(2, idOffset);
    }
    if (currencyCodeOffset != null) {
      fbBuilder.addOffset(3, currencyCodeOffset);
    }
    fbBuilder.addInt64(4, _amount);
    fbBuilder.addUint32(5, _decimals);
    if (authorityOffset != null) {
      fbBuilder.addOffset(6, authorityOffset);
    }
    if (issuerOffset != null) {
      fbBuilder.addOffset(7, issuerOffset);
    }
    fbBuilder.addInt64(8, _issuanceDate);
    if (authorizedSignatureOffset != null) {
      fbBuilder.addOffset(9, authorizedSignatureOffset);
    }
    if (securedSignatureOffset != null) {
      fbBuilder.addOffset(10, securedSignatureOffset);
    }
    if (signatureBlockOffset != null) {
      fbBuilder.addOffset(11, signatureBlockOffset);
    }
    fbBuilder.addUint8(12, _blockDepth);
    return fbBuilder.endTable();
  }

  /// Convenience method to serialize to byte list.
  @override
  Uint8List toBytes([String fileIdentifier = "MONE"]) {
    fb.Builder fbBuilder = new fb.Builder();
    int offset = finish(fbBuilder);
    fbBuilder.finish(offset, fileIdentifier);
    return fbBuilder.buffer;
  }
}
