import 'dart:ffi' as ffi;
import 'dart:io' show Platform, Directory;
import 'dart:convert';
import 'dart:typed_data';
import 'dart:developer' as dev;

import 'package:ffi/ffi.dart';
import 'package:flat_buffers/flat_buffers.dart';
import 'package:path/path.dart' as path;

import "./monetae_ffi.dart";
import "./wallet.dart";
import "./utils.dart";

/// Proxy for connecting to gateway networks
class Proxy {
  late ffi.Pointer<CProxy> _inner;
  late NativeLibrary _lib;

  // Initialize wallet instance from 32 byte seed
  Proxy(Wallet wallet, List<String> endpoints) {
    ffi.DynamicLibrary dylib = ffi.DynamicLibrary.open(
        path.join(Directory.current.path, "dylib/libmonetae_ffi.dylib"));

    _lib = NativeLibrary(dylib);

    // convert endpoints list into C layout
    final _endpoints = uint8ListToVecUint8Pointer(
        Uint8List.fromList(utf8.encode(endpoints.join(","))));

    _inner = _lib.proxy_connect(
        wallet.ptr(), _endpoints.ref, ffi.Pointer.fromFunction(receivePacket));
  }

  // process packet
  static void receivePacket(CPacket packet) {
    print("Received Packet in Dart: $packet");
  }

  // Send a packet to the proxy
  void sendPacket(CPacket packet) {
    // TODO: Change CPacket to protocol buffer type and convert to bytes for CPacket
    _lib.proxy_send(_inner, packet);
  }
}
