import 'dart:ffi' as ffi;
import 'dart:io' show Platform, Directory;
import 'dart:convert';
import 'dart:typed_data';
import 'dart:developer' as dev;

import 'package:ffi/ffi.dart';
import 'package:flat_buffers/flat_buffers.dart';
import 'package:path/path.dart' as path;

import "./monetae_ffi.dart";
import "./utils.dart";

class Wallet {
  late ffi.Pointer<CWallet> _inner;
  late NativeLibrary _lib;

  // Initialize wallet instance from 32 byte seed
  Wallet(Uint8List seed) {
    ffi.DynamicLibrary dylib = ffi.DynamicLibrary.open(
        path.join(Directory.current.path, "dylib/libmonetae_ffi.dylib"));

    _lib = NativeLibrary(dylib);
    ffi.Pointer<slice_ref_uint8_t> _seed = uint8ListToSliceUint8Pointer(seed);
    _inner = _lib.wallet_from_seed(_seed.ref);

    // Free seed pointer after wallet pointer is created
    calloc.free(_seed);
  }

  // return the verifier key for the wallet
  Uint8List verifier() {
    final slice_ref_uint8_t verifier = _lib.wallet_verifier(_inner);

    return uint8SliceToUint8List(verifier);
  }

  // Sign a payload and return the signature
  Uint8List sign(Uint8List payload) {
    final ffi.Pointer<slice_ref_uint8_t> _payload =
        uint8ListToSliceUint8Pointer(payload);
    final Vec_uint8_t signature = _lib.wallet_sign(_inner, _payload.ref);

    return vecUint8ToUint8List(signature);
  }

  // Verify a signature given a verifier key, payload and signature
  bool verify(Uint8List verifier, Uint8List payload, Uint8List signature) {
    final ffi.Pointer<slice_ref_uint8_t> _verifier =
        uint8ListToSliceUint8Pointer(verifier);
    final ffi.Pointer<slice_ref_uint8_t> _payload =
        uint8ListToSliceUint8Pointer(payload);
    final ffi.Pointer<slice_ref_uint8_t> _signature =
        uint8ListToSliceUint8Pointer(signature);

    final bool result =
        _lib.wallet_verify_ed25519(_verifier.ref, _payload.ref, _signature.ref);

    return result;
  }

  ffi.Pointer<CWallet_t> ptr() {
    return _inner;
  }
}
