import 'dart:typed_data';
import 'dart:convert';
import 'dart:io';

import 'package:knox_sdk/knox_sdk.dart';
import 'package:test/test.dart';

const List<int> SECRET_KEY = [
  79,
  112,
  101,
  110,
  32,
  83,
  111,
  117,
  114,
  99,
  101,
  32,
  77,
  111,
  110,
  101,
  121,
  32,
  102,
  111,
  114,
  32,
  116,
  104,
  101,
  32,
  77,
  97,
  115,
  115,
  101,
  115
];

const List<int> VERIFIER_KEY = [
  68,
  231,
  237,
  15,
  62,
  30,
  83,
  40,
  52,
  199,
  228,
  191,
  40,
  172,
  146,
  63,
  236,
  106,
  56,
  152,
  228,
  61,
  84,
  120,
  203,
  71,
  104,
  9,
  243,
  146,
  221,
  81
];

void main() {
  group('Test promissory compatibility with rust code', () {
    // read promissory file from local directory

    setUp(() {
      // Additional setup goes here.
    });

    test('Verify promissory', () {
      // expect(awesome.isAwesome, isTrue);
    });
  });

  group('Test wallet compatibility with rust code', () {
    setUp(() {
      // Additional setup goes here.
    });

    test('Wallet Verifier', () {
      try {
        final wallet = Wallet(Uint8List.fromList(SECRET_KEY));

        // Expect wallet verifier to equal the value of `VERIFIER_KEY`
        expect(wallet.verifier(), Uint8List.fromList(VERIFIER_KEY));
      } catch (e) {
        print(e);
      }
    });

    test('Wallet signer', () {
      try {
        final wallet = Wallet(Uint8List.fromList(SECRET_KEY));

        // Utf8 bytes message payload to sign
        final payload = Uint8List.fromList(utf8.encode('Hello, Rust!'));
        final signature = wallet.sign(payload);

        print("Found Signature: $signature");

        // Expect signature length to be 64 bytes
        expect(signature.length, 64);
      } catch (e) {
        print(e);
      }
    });

    test('Wallet verify signature', () {
      try {
        final wallet = Wallet(Uint8List.fromList(SECRET_KEY));

        // Utf8 bytes message payload to sign
        final payload = Uint8List.fromList(utf8.encode('Hello, Rust!'));
        final signature = wallet.sign(payload);

        // Expect signature length to be 64 bytes
        expect(wallet.verify(wallet.verifier(), payload, signature), true);
      } catch (e) {
        print(e);
      }
    });
  });

  group('Test proxy compatibility with rust code', () {
    setUp(() {
      // Additional setup goes here.
    });

    test('Proxy Connection', () {
      List<String> endpoints = ["http://0.0.0.0:8796"];
      final wallet = Wallet(Uint8List.fromList(SECRET_KEY));

      final proxy = Proxy(wallet, endpoints);

      int counter = 10;

      while (counter != 0) {
        // Sleep for 10 seconds
        sleep(Duration(seconds: 1));

        counter--;
      }
    });
  });
}
